@@ -9,8 +9,9 @@ import javax.swing.JComponent;
 import javax.net.ssl.HttpsURLConnection;
 
 /**
- *
- * @author Mike
+ * DEMReader - an abstract base class  for readers of various forms of Digital Elevation Model data sets
+ * 
+ * @author Mike McCombe
  */
 public abstract class DEMReader extends PropertyChangeSupport {
 
@@ -142,7 +143,6 @@ public abstract class DEMReader extends PropertyChangeSupport {
         }
 // Record not in cache or cache not enabled
         try {
-
             String filename = DIRECTORY + name + extn();
             File infile = new File(filename);
             if (!infile.isFile()) {
@@ -159,11 +159,21 @@ public abstract class DEMReader extends PropertyChangeSupport {
             }
             in = new java.util.zip.ZipInputStream(new FileInputStream(infile));
             String entryname = "";
+            String hgtentryname = null;
             do {
                 ZipEntry entry = in.getNextEntry();
                 if(entry==null) throw new MissingDataFileException(String.format("ZIP file %s does not contain expected entry %s",filename,zipEntryName(name)));
                 entryname = entry.getName();
             } while (!entryname.equalsIgnoreCase(zipEntryName(name)));
+//            while (entry!=null){ 
+//                entryname = entry.getName();
+//                if(entryname.equalsIgnoreCase(zipEntryName(name))){
+//                    hgtentryname = entryname ;
+//                }
+//                entry = in.getNextEntry();
+//            } 
+//            if(hgtentryname==null) throw new MissingDataFileException(String.format("ZIP file %s does not contain expected entry %s",filename,zipEntryName(name)));
+//            //
             int recordno = 0;
             try {
                 int[] heights;
@@ -202,8 +212,13 @@ public abstract class DEMReader extends PropertyChangeSupport {
             throw new MissingDataFileException("Unable to read file - " + e.toString());
         }
     }
-
-    public int[] readRecord() throws EOFException, IOException {
+/**
+ * Read the next record from the current ZipInputStream
+ * 
+ * @return int[] buffer of length recordlength() containing the decoded next record;
+ * @throws IOException 
+ */
+    public int[] readRecord() throws IOException {
         byte[] buffer = new byte[recordlength() * 2];
         int[] outbuffer = new int[recordlength()];
         int sofar = 0;
@@ -211,49 +226,94 @@ public abstract class DEMReader extends PropertyChangeSupport {
         while (sofar < recordlength() * 2) {
             int res = in.read(buffer, sofar, recordlength() * 2 - sofar);
             if (res == -1) {
-                throw new EOFException();
+                break;
             }
             sofar += res;
         }
         for (int i = 0; i < recordlength(); i++) {
             short temp;
-            if (littleendian()) {
-                temp = (short) (buffer[2 * i + 1] << 8 | (0xff & buffer[2 * i]));
-            } else {
-                temp = (short) (buffer[2 * i] << 8 | (0xff & buffer[2 * i + 1]));
+            if(2 * i < sofar) {
+                if (littleendian()) {
+                    temp = (short) (buffer[2 * i + 1] << 8 | (0xff & buffer[2 * i]));
+                } else {
+                    temp = (short) (buffer[2 * i] << 8 | (0xff & buffer[2 * i + 1]));
+                }
+                outbuffer[i] = temp;
+            }
+            else {
+                outbuffer[i] = missingValue();
             }
-            outbuffer[i] = temp;
         }
         return outbuffer;
     }
-
-    public double tile(double x) {
+/**
+ * Read the next byte record from the current ZipInputStream
+ * 
+ * @return byte[] buffer of length recordlength() containing the decoded next record;
+ * @throws IOException 
+ */
+    public byte[] readByteRecord() throws IOException {
+        byte[] buffer = new byte[recordlength()];
+        int sofar = 0;
+        //Keep reading until we have the whole record
+        while (sofar < recordlength()) {
+            int res = in.read(buffer, sofar, recordlength() - sofar);
+            if (res == -1) {
+                break;
+            }
+            sofar += res;
+        }
+       return buffer;
+    }
+    /**
+     * Calculate the number of the element in the buffer that refers to a particular lat or lon value
+     * 
+     * @param x the double lat/lon value
+     * @return A double value representing the buffer element index
+     */
+        public double tile(double x) {
         double q = Math.floor(x);
         double r = (recordlength() - 1) * (x - q);
         return r;
     }
-
+    protected String getDownloadSiteName(){
+        return null;
+    }
+    /**
+     * Calculate the proportion of a degree represented by a particular element in the buffer
+     * 
+     * @param tile the integer number of the element
+     * @return double number representing a fraction of a degree
+     */
     public double frac(int tile) {
         double x = (double) tile / (double) (recordlength() - 1);
         return x;
     }
-
+    /**
+     * Download a specific file and sand save it in scratch space on the local machine
+     * 
+     * @param filename the name of the file to download
+     * @throws java.security.KeyManagementException
+     * @throws java.security.NoSuchAlgorithmException
+     * @throws IOException
+     * @throws MissingDataFileException 
+     */
     protected void downloadFile(String filename) throws java.security.KeyManagementException, java.security.NoSuchAlgorithmException, IOException, MissingDataFileException {
-        if (!download || !downloadable()) {
+        String sitename = getDownloadSiteName();
+        if (!download || !downloadable() || sitename==null) {
 
             String msg = String.format("Needs data file %s - DEM not downloadable or auto-download is disabled", filename);
             throw new MissingDataFileException(msg);
         }
-        String urlString = getProperty(TerrainProperties.FTP) + getProperty(TerrainProperties.REGION) + "/" + filename;
+//        String urlString = getProperty(TerrainProperties.FTP) + getProperty(TerrainProperties.REGION) + "/" + filename;
+        String urlString = getDownloadSiteName() + "/" + filename;
         java.net.URL url = new java.net.URL(urlString);
         setMessage(String.format("Downloading data: %s", filename));
         HttpsURLConnection con = (HttpsURLConnection)url.openConnection();
         InputStream ins = con.getInputStream();
-//        java.io.DataInputStream instream = new java.io.DataInputStream(ins);
         String outfilename = DIRECTORY + filename;
         File outfile = new java.io.File(outfilename);
         DataOutputStream outstream = new DataOutputStream(new FileOutputStream(outfile));
-        long available = ins.available();
         int bytecount = 0;
         long sofar = 0;
         long guess = 900000;
@@ -272,28 +332,48 @@ public abstract class DEMReader extends PropertyChangeSupport {
         outstream.close();
         ins.close();
         setMessage("");
-        return;
     }
+/**
+ * CacheEntry -- A class used to store a row of values 
+ */
     protected class CacheEntry {
-
+/**
+ * Create a CacheEntry object which contains a named row of the current tile.
+ * 
+ * @param name The name of the row in the form <tileName>#<rownumber> e.g. "NASADEM_HGT_N51W003#1034"
+ * @param buffer an integer buffer of length recordlength() containing the heights for the specified row.
+ */
         public CacheEntry(String name, int[] buffer) {
             page_name = name;
             last_used = cycle;
             data = buffer;
         }
-
+/**
+ * 
+ */
         public void setLastUsed() {
             last_used = cycle;
         }
-
+/**
+ * getName() Get the name of the CacheEntry
+ * 
+ * @return String the name of this CacheEntry
+ */
         public String getName() {
             return page_name;
         }
-
+/**
+ * 
+ * @param i int the index value of the item in the row to fetch
+ * @return int the value of the specified element
+ */
         public int getValue(int i) {
             return data[i];
         }
-
+/**
+ * Find when this CacheEntry was last referenced
+ * @return long the cycle number when the entry was last referenced
+ */
         public long lastUsed() {
             return last_used;
         }
@@ -301,7 +381,12 @@ public abstract class DEMReader extends PropertyChangeSupport {
         private String page_name;
         private int[] data;
     }
-
+/**
+ * Fit an exact polynomial to a set of points and return its value at a specified point
+ * @param x double the value at which to evaluate the calculated polynomial
+ * @param points a set of java.awt.geom.Point2D.Double values
+ * @return double value of the polynomial
+ */
     public static double lagrangian(double x, java.awt.geom.Point2D.Double... points) {
         int n = points.length;
         double tot = 0.0;
@@ -340,14 +425,14 @@ public abstract class DEMReader extends PropertyChangeSupport {
         resultcount = 0;
     }
 
-    private void setMessage(String msg) {
+    protected void setMessage(String msg) {
         if (!msg.equals(lastMessage)) {
             firePropertyChange("message", lastMessage, msg);
         }
         lastMessage = msg;
     }
 
-    private void setProgress(int val) {
+    protected void setProgress(int val) {
         int v = Math.min(100, val);
         v = Math.max(0, v);
         if (lastValue != v) {
@@ -356,7 +441,7 @@ public abstract class DEMReader extends PropertyChangeSupport {
         lastValue = v;
     }
 
-    private String getProperty(TerrainProperties propertyName) {
+    protected String getProperty(TerrainProperties propertyName) {
         return TerrainFrame.properties.get(propertyName);
     }
 
@@ -382,42 +467,39 @@ public abstract class DEMReader extends PropertyChangeSupport {
             }
         }
     };
-    public void setLegacy(boolean t) {
-        useLegacy = t ;
-    }
+//    public void setLegacy(boolean t) {
+//        useLegacy = t ;
+//    }
     public abstract String datasetName();
-
     public abstract boolean downloadable();
-
     public abstract int recordlength();
-
     public abstract String formatstring();
-
     public abstract String extn();
-
     public abstract boolean littleendian();
-
     public abstract int missingValue();
-    public abstract String copyright();
-
-    public abstract String zipEntryName(String name);
+    public abstract String[] copyright() ;
+    public String zipEntryName(String name){
+        return name + ".hgt";
+    }
+    public String numEntryName(String name){
+        return "";
+    }
     protected static java.util.zip.ZipInputStream in;
     protected  String DIRECTORY = "";
-//    private static final int DEMSettings.recordlength() = 1201;
     protected long cycle = 0;
-    protected java.util.HashMap<String, CacheEntry> cache = new java.util.HashMap<String, CacheEntry>();
+    protected java.util.HashMap<String, CacheEntry> cache = new java.util.HashMap<>();
     protected boolean cacheEnable = true;
     protected static final int MAX_CACHE_SIZE = 16;
     protected long hits = 0;
     protected long tries = 0;
     protected long missing = 0;
     protected long resultcount = 0;
-    public static final double MISSING = -32768.0;
+    protected static final double MISSING = -32768.0;
     protected static final int BUFFERLENGTH = 1024;  
     protected String lastMessage = "";
     protected boolean download = true;
     protected int lastValue = 0;
     protected static final java.util.Locale LOCALE = java.util.Locale.UK; //Force use of UK locale for number formatting
     protected boolean useLegacy = false ;
-//    protected DEMProfile DEMSettings = DEMProfile.SRTM;
+    protected String[] copyright = {"Copyright statement is missing"};
 }
