@@ -34,9 +34,8 @@ public class DefaultProperties extends java.util.Properties {
         this.setProperty("latitude","52.0");
         this.setProperty("longitude","-2.0");
         this.setProperty("locale", "GB");
-        this.setProperty("useASTER", "srtm");
+        this.setProperty("useASTER", "nasa");
         this.setProperty("therionCoordinateSet","OSGB");
-        this.setProperty("legacyASTER","false");
    }
 }
 
