@@ -50,13 +50,13 @@ public class MosaicPanel extends JPanel {
      * @param buffer - a double[][] array containing the map data. First
      * dimension is "northing", second is "easting"
      */
-    public void setDataTable(float[][] buffer) {
+    public void setDataTable(float[][] buffer, DEMReader reader) {
         dataTable = buffer;
         minval = 1.0E99;
         maxval = -1.0E99;
         for (float[] row : dataTable) {
             for (float x : row) {
-                if (x != SRTM2Reader.MISSING) {
+                if (x != reader.missingValue()) {
                     minval = Math.min(minval, x);
                     maxval = Math.max(maxval, x);
                 }
@@ -114,7 +114,7 @@ public class MosaicPanel extends JPanel {
      * @return the corresponding Color
      */
     private Color selectColour(float altitude) {
-        if (altitude == 0.0) {
+        if (altitude <= 0.0) {
             return colours[0];
         }
         if (altitude == SRTM2Reader.MISSING) {
